@@ -11,6 +11,7 @@ package Kernel::Modules::AgentTicketCompose;
 
 use strict;
 use warnings;
+use utf8;
 
 use Kernel::System::VariableCheck qw(:all);
 use Kernel::Language qw(Translatable);
@@ -66,9 +67,10 @@ sub Run {
     }
 
     # get needed objects
-    my $ConfigObject  = $Kernel::OM->Get('Kernel::Config');
-    my $TicketObject  = $Kernel::OM->Get('Kernel::System::Ticket');
-    my $ArticleObject = $Kernel::OM->Get('Kernel::System::Ticket::Article');
+    my $ConfigObject    = $Kernel::OM->Get('Kernel::Config');
+    my $TicketObject    = $Kernel::OM->Get('Kernel::System::Ticket');
+    my $ArticleObject   = $Kernel::OM->Get('Kernel::System::Ticket::Article');
+    my $HTMLUtilsObject = $Kernel::OM->Get('Kernel::System::HTMLUtils');
 
     my $ArticleBackendObject = $ArticleObject->BackendForChannel( ChannelName => 'Email' );
 
@@ -836,22 +838,25 @@ sub Run {
                 }
             }
 
-            my $ValidationResult = $DynamicFieldBackendObject->EditFieldValueValidate(
-                DynamicFieldConfig   => $DynamicFieldConfig,
-                PossibleValuesFilter => $PossibleValuesFilter,
-                ParamObject          => $ParamObject,
-                Mandatory =>
-                    $Config->{DynamicField}->{ $DynamicFieldConfig->{Name} } == 2,
-            );
-
-            if ( !IsHashRefWithData($ValidationResult) ) {
-                return $LayoutObject->ErrorScreen(
-                    Message => $LayoutObject->{LanguageObject}->Translate(
-                        'Could not perform validation on field %s!',
-                        $DynamicFieldConfig->{Label},
-                    ),
-                    Comment => Translatable('Please contact the administrator.'),
+            my $ValidationResult;
+            if ( $Self->{LoadedFormDraftID} && $Self->{Subaction} && $Self->{Subaction} ne '' ) {
+                $ValidationResult = $DynamicFieldBackendObject->EditFieldValueValidate(
+                    DynamicFieldConfig   => $DynamicFieldConfig,
+                    PossibleValuesFilter => $PossibleValuesFilter,
+                    ParamObject          => $ParamObject,
+                    Mandatory =>
+                        $Config->{DynamicField}->{ $DynamicFieldConfig->{Name} } == 2,
                 );
+
+                if ( !IsHashRefWithData($ValidationResult) ) {
+                    return $LayoutObject->ErrorScreen(
+                        Message => $LayoutObject->{LanguageObject}->Translate(
+                            'Could not perform validation on field %s!',
+                            $DynamicFieldConfig->{Label},
+                        ),
+                        Comment => Translatable('Please contact the administrator.'),
+                    );
+                }
             }
 
             # propagate validation error to the Error variable to be detected by the frontend
@@ -892,12 +897,18 @@ sub Run {
 
             # When a draft is loaded, inform a user that article subject will be empty
             # if contains only the ticket hook (if nothing is modified).
-            if ( $Error{LoadedFormDraft} ) {
-                $Output .= $LayoutObject->Notify(
-                    Data => $LayoutObject->{LanguageObject}->Translate(
-                        'Article subject will be empty if the subject contains only the ticket hook!'
-                    ),
+            if ( $Error{LoadedFormDraft} && IsStringWithData( $GetParam{Subject} ) ) {
+                my $CleanedSubject = $TicketObject->TicketSubjectClean(
+                    TicketNumber => $Ticket{TicketNumber},
+                    Subject      => $GetParam{Subject},
                 );
+                if ( !IsStringWithData($CleanedSubject) ) {
+                    $Output .= $LayoutObject->Notify(
+                        Data => $LayoutObject->{LanguageObject}->Translate(
+                            'Article subject will be empty if the subject contains only the ticket hook!'
+                        ),
+                    );
+                }
             }
 
             $GetParam{StandardResponse} = $GetParam{Body};
@@ -1295,6 +1306,36 @@ sub Run {
             NoCache     => 1,
         );
     }
+    elsif ( $Self->{Subaction} eq 'CheckSubject' ) {
+
+        # Inform a user that article subject will be empty if contains only the ticket hook (if nothing is modified).
+        my $Message = $LayoutObject->{LanguageObject}->Translate(
+            'Article subject will be empty if the subject contains only the ticket hook!'
+        );
+
+        my $Subject = $ParamObject->GetParam( Param => 'Subject' );
+
+        my $CleanedSubject = $TicketObject->TicketSubjectClean(
+            TicketNumber => $Ticket{TicketNumber},
+            Subject      => $Subject,
+        );
+
+        my $Empty = !length $CleanedSubject;
+
+        my $JSON = $LayoutObject->JSONEncode(
+            Data => {
+                Empty   => $Empty ? 1 : 0,
+                Message => $Message,
+            }
+        );
+
+        return $LayoutObject->Attachment(
+            ContentType => 'application/json; charset=' . $LayoutObject->{Charset},
+            Content     => $JSON,
+            Type        => 'inline',
+            NoCache     => 1,
+        );
+    }
     else {
         my $Output = $LayoutObject->Header(
             Value     => $Ticket{TicketNumber},
@@ -1302,13 +1343,6 @@ sub Run {
             BodyClass => 'Popup',
         );
 
-        # Inform a user that article subject will be empty if contains only the ticket hook (if nothing is modified).
-        $Output .= $LayoutObject->Notify(
-            Data => $LayoutObject->{LanguageObject}->Translate(
-                'Article subject will be empty if the subject contains only the ticket hook!'
-            ),
-        );
-
         # get std attachment object
         my $StdAttachmentObject = $Kernel::OM->Get('Kernel::System::StdAttachment');
 
@@ -1446,7 +1480,7 @@ sub Run {
             UploadCacheObject => $UploadCacheObject,
         );
 
-        my %SafetyCheckResult = $Kernel::OM->Get('Kernel::System::HTMLUtils')->Safety(
+        my %SafetyCheckResult = $HTMLUtilsObject->Safety(
             String => $Data{Body},
 
             # Strip out external content if BlockLoadingRemoteContent is enabled.
@@ -1462,23 +1496,17 @@ sub Run {
         $Data{Body} = $SafetyCheckResult{String};
 
         # restrict number of body lines if configured
+        my $ResponseQuoteMaxLines = $ConfigObject->Get('Ticket::Frontend::ResponseQuoteMaxLines');
         if (
-            $Data{Body}
-            && $ConfigObject->Get('Ticket::Frontend::ResponseQuoteMaxLines')
+            IsStringWithData( $Data{Body} )
+            && $ResponseQuoteMaxLines
             )
         {
-            my $MaxLines = $ConfigObject->Get('Ticket::Frontend::ResponseQuoteMaxLines');
-
-            # split body - one element per line
-            my @Body = split "\n", $Data{Body};
-
-            # only modify if body is longer than allowed
-            if ( scalar @Body > $MaxLines ) {
-
-                # splice to max. allowed lines and reassemble
-                @Body = @Body[ 0 .. ( $MaxLines - 1 ) ];
-                $Data{Body} = join "\n", @Body;
-            }
+            $Data{Body} = $HTMLUtilsObject->TruncateBodyQuote(
+                Body       => $Data{Body},
+                Limit      => $ResponseQuoteMaxLines,
+                HTMLOutput => $LayoutObject->{BrowserRichText},
+            ) // $Data{Body};
         }
 
         if ( $LayoutObject->{BrowserRichText} ) {
@@ -1487,6 +1515,7 @@ sub Run {
             # rewrap body if exists
             if ( $Data{Body} ) {
                 $Data{Body} =~ s/\t/ /g;
+
                 my $Quote = $LayoutObject->Ascii2Html(
                     Text           => $ConfigObject->Get('Ticket::Frontend::Quote') || '',
                     HTMLResultMode => 1,
@@ -1535,6 +1564,7 @@ sub Run {
             # re-wrap body if exists
             if ( $Data{Body} ) {
                 $Data{Body} =~ s/\t/ /g;
+
                 my $Quote = $ConfigObject->Get('Ticket::Frontend::Quote');
                 if ($Quote) {
                     $Data{Body} =~ s/\n/\n$Quote /g;
@@ -1942,9 +1972,9 @@ sub _Mask {
         OnlyDynamicFields => 1
     );
 
-    # get needed objects
-    my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
-    my $ConfigObject = $Kernel::OM->Get('Kernel::Config');
+    my $LayoutObject    = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
+    my $ConfigObject    = $Kernel::OM->Get('Kernel::Config');
+    my $FormDraftObject = $Kernel::OM->Get('Kernel::System::FormDraft');
 
     # get config for frontend module
     my $Config = $ConfigObject->Get("Ticket::Frontend::$Self->{Action}");
@@ -2253,20 +2283,9 @@ sub _Mask {
 
     # show time accounting box
     if ( $ConfigObject->Get('Ticket::Frontend::AccountTime') ) {
-        if ( $ConfigObject->Get('Ticket::Frontend::NeedAccountedTime') ) {
-            $LayoutObject->Block(
-                Name => 'TimeUnitsLabelMandatory',
-                Data => \%Param,
-            );
-            $Param{TimeUnitsRequired} = 'Validate_Required';
-        }
-        else {
-            $LayoutObject->Block(
-                Name => 'TimeUnitsLabel',
-                Data => \%Param,
-            );
-            $Param{TimeUnitsRequired} = '';
-        }
+        $Param{TimeUnitsBlock} = $LayoutObject->TimeUnits(
+            %Param,
+        );
         $LayoutObject->Block(
             Name => 'TimeUnits',
             Data => \%Param,
@@ -2318,7 +2337,7 @@ sub _Mask {
 
     my $LoadedFormDraft;
     if ( $Self->{LoadedFormDraftID} ) {
-        $LoadedFormDraft = $Kernel::OM->Get('Kernel::System::FormDraft')->FormDraftGet(
+        $LoadedFormDraft = $FormDraftObject->FormDraftGet(
             FormDraftID => $Self->{LoadedFormDraftID},
             GetContent  => 0,
             UserID      => $Self->{UserID},
@@ -2364,15 +2383,24 @@ sub _Mask {
         );
     }
 
+    # Check if the user has already any form draft for this action
+    my $FormDraftList = $FormDraftObject->FormDraftListGet(
+        ObjectType => 'Ticket',
+        ObjectID   => $Self->{TicketID},
+        Action     => $Self->{Action},
+        UserID     => $Self->{UserID},
+    ) // [];
+
     # create & return output
     return $LayoutObject->Output(
         TemplateFile => 'AgentTicketCompose',
         Data         => {
-            FormID         => $Self->{FormID},
-            FormDraft      => $Config->{FormDraft},
-            FormDraftID    => $Self->{LoadedFormDraftID},
-            FormDraftTitle => $LoadedFormDraft ? $LoadedFormDraft->{Title} : '',
-            FormDraftMeta  => $LoadedFormDraft,
+            FormID             => $Self->{FormID},
+            FormDraft          => $Config->{FormDraft},
+            FormDraftID        => $Self->{LoadedFormDraftID},
+            FormDraftTitle     => $LoadedFormDraft ? $LoadedFormDraft->{Title} : '',
+            FormDraftMeta      => $LoadedFormDraft,
+            FormDraftForAction => scalar @{$FormDraftList},
             %Param,
         },
     );
